import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/foundation.dart';
import 'packet.dart';

// 检查数据长度是否合理 (可以设置一个最大值以防止异常)
const int maxDataLength = 1024 + 7;

/// 处理接收蓝牙数据的解析器，将字节流解析为数据包结构

class Parser {
  /// 接收数据缓冲区
  final List<int> _buffer = [];

  final Function(Packet) onParser; // 回调函数

  Parser(this.onParser);


  /// 添加数据到缓冲区并尝试解析
  void add(List<int> data) {
    // 添加数据到缓冲区
    _buffer.addAll(data);
    // 尝试解析数据包
    _processBuffer();
  }

  /// 处理缓冲区，尝试解析出完整的数据包
  void _processBuffer() {
    // 包头需要至少5个字节
    while (_buffer.length >= 7) {
      // 检查包类型是否有效
      int typeValue = _buffer[0];
      if (typeValue < 0x01 || typeValue > 0x06) {
        // 无效包类型，丢弃当前字节并继续
        _buffer.removeAt(0);
        continue;
      }

      // 获取包类型
      PacketType type = PacketType.fromValue(typeValue);

      // 解析序列号和数据长度
      int seqNum = (_buffer[1] << 8) | _buffer[2];
      int dataLen = (_buffer[3] << 8) | _buffer[4];

      // 检查数据长度是否合理 (可以设置一个最大值以防止异常)
      if (dataLen < 0 || dataLen > maxDataLength) {
        // 假设最大数据长度为1KB
        // 数据长度异常，丢弃当前字节并继续
        _buffer.removeAt(0);
        continue;
      }

      // 检查缓冲区是否有足够的数据 (包头5字节 + 数据长度 + CRC 2字节)
      int totalPacketLength = 5 + dataLen + 2;
      if (_buffer.length < totalPacketLength) {
        // 数据不完整，等待更多数据
        break;
      }

      // 提取完整的数据包
      List<int> packetBytes = _buffer.sublist(0, totalPacketLength);

      // 验证CRC
      int expectedCrc = (packetBytes[totalPacketLength - 2] << 8) | packetBytes[totalPacketLength - 1];
      List<int> dataToCheck = packetBytes.sublist(0, totalPacketLength - 2);
      int calculatedCrc = calculateCrc16(Uint8List.fromList(dataToCheck));

      if (calculatedCrc != expectedCrc) {
        // CRC校验失败，丢弃当前字节并继续
        _buffer.removeAt(0);
        continue;
      }

      // CRC校验通过，创建数据包对象
      PacketHeader header = PacketHeader(type, seqNum, dataLen);
      Uint8List data = Uint8List.fromList(_buffer.sublist(5, 5 + dataLen));
      Packet packet = Packet(header, data, expectedCrc);

      // 发送解析出的数据包
      onParser(packet); // 回调函数

      // 从缓冲区移除已处理的数据
      _buffer.removeRange(0, totalPacketLength);
    }
  }

  /// 计算CRC16校验值
  int calculateCrc16(Uint8List data) {
    int crc = 0;
    for (int byte in data) {
      crc = (crc ^ byte) & 0xFFFF;
      for (int i = 0; i < 8; i++) {
        if ((crc & 0x0001) != 0) {
          crc = ((crc >> 1) ^ 0xA001) & 0xFFFF;
        } else {
          crc = (crc >> 1) & 0xFFFF;
        }
      }
    }
    return crc;
  }

  /// 创建开始包
  Packet createStartPacket(int fileSize, int blockSize, String fileName) {
    // 创建数据: [文件大小(4字节) + 块大小(2字节) + 文件名(n字节)]
    // 文件大小: 高位在前
    List<int> data = [
      (fileSize >> 24) & 0xFF,
      (fileSize >> 16) & 0xFF,
      (fileSize >> 8) & 0xFF,
      fileSize & 0xFF,
      (blockSize >> 8) & 0xFF,
      blockSize & 0xFF,
    ];

    // 添加文件名的ASCII码
    data.addAll(fileName.codeUnits);

    // 创建包头
    PacketHeader header = PacketHeader(PacketType.start, 0, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes, ...data]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, Uint8List.fromList(data), crc);
  }

  /// 创建数据包
  Packet createDataPacket(int seqNum, Uint8List data) {
    // 创建包头
    PacketHeader header = PacketHeader(PacketType.data, seqNum, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes, ...data]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, data, crc);
  }

  /// 创建确认包
  Packet createAckPacket(int seqNum) {
    // 确认包的数据部分包含被确认的包序号
    Uint8List data = Uint8List(2);
    data[0] = (seqNum >> 8) & 0xFF;
    data[1] = seqNum & 0xFF;

    // 创建包头
    PacketHeader header = PacketHeader(PacketType.ack, seqNum, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes, ...data]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, data, crc);
  }

  /// 创建重传请求包
  Packet createNakPacket(int seqNum) {
    // 重传请求包的数据部分包含需要重传的包序号
    Uint8List data = Uint8List(2);
    data[0] = (seqNum >> 8) & 0xFF;
    data[1] = seqNum & 0xFF;

    // 创建包头
    PacketHeader header = PacketHeader(PacketType.nak, seqNum, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes, ...data]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, data, crc);
  }

  /// 创建结束包
  Packet createEndPacket(int seqNum) {
    // 结束包的数据部分为空
    Uint8List data = Uint8List(0);

    // 创建包头
    PacketHeader header = PacketHeader(PacketType.end, seqNum, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, data, crc);
  }

  /// 创建错误包
  Packet createErrorPacket(int errorCode) {
    // 错误包的数据部分包含错误码
    Uint8List data = Uint8List(1);
    data[0] = errorCode & 0xFF;

    // 创建包头 (错误包序号默认为0)
    PacketHeader header = PacketHeader(PacketType.error, 0, data.length);

    // 计算CRC
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value;
    headerBytes[1] = (header.seqNum >> 8) & 0xFF;
    headerBytes[2] = header.seqNum & 0xFF;
    headerBytes[3] = (header.dataLen >> 8) & 0xFF;
    headerBytes[4] = header.dataLen & 0xFF;

    Uint8List dataToCheck = Uint8List.fromList([...headerBytes, ...data]);
    int crc = calculateCrc16(dataToCheck);

    return Packet(header, data, crc);
  }

  /// 解析开始包中的数据
  Map<String, dynamic> parseStartPacketData(Uint8List data) {
    // 文件大小: 4字节
    int fileSize = (data[0] << 24) | (data[1] << 16) | (data[2] << 8) | data[3];

    // 块大小: 2字节
    int blockSize = (data[4] << 8) | data[5];

    // 文件名: 剩余字节
    String fileName = String.fromCharCodes(data.sublist(6));

    return {
      'fileSize': fileSize,
      'blockSize': blockSize,
      'fileName': fileName,
    };
  }

  /// 解析确认包和重传请求包中的序列号
  int parseAckNakPacketData(Uint8List data) {
    // 序列号: 2字节
    return (data[0] << 8) | data[1];
  }

  /// 解析错误包中的错误码
  int parseErrorPacketData(Uint8List data) {
    // 错误码: 1字节
    return data[0];
  }

  /// 重置解析器
  void reset() => _buffer.clear();
}
